#############################################################################  
#                                                                           #
# Fichiers de dfinition des variations morphologiques                      #
#                                                                           #
#############################################################################  

# tableau contenant les variations morphologiques
# pour chaque suffixe d'adjectif en premier champ, le tableau fournit
# le suffixe de nom equivalent en deuxieme champ 

$E = "[]";  

$CONSON = "[trndplmbvgfh]";  


%MorphAdjRel = (
    ique =>  \@Regleique ,
    aire =>  \@Regleaire,
    al   =>  \@Regleal,
    eux  =>  \@Regleeux,
    oire  =>  \@Regleoire,
    ""  =>  \@Regleeaigu, # Traiter dans Fonction Morphologie directement
    cole  =>  \@Reglecole,
    er  =>  \@Regleer,
    ien  =>  \@Regleien,
    ain  =>  \@Regleain,
    el  =>  \@Regleel,
    in  =>  \@Reglein,
    if  =>  \@Regleif
    ) ;

@Regleique = ( 
["matique","me"],
["trique","tre"],
["tifique","ce"],
["otique","ose"],          #hypnotique/hypnose
["inique","en"],
["atique","e"],         # suffixe en "ique"
["atique","a"],         
["lique","ile"],
["cique","x"],
["tique",""], 
["ique",""],
["ique","ie"],
["ique","e"],            #dictionnaire/dictionnairique
["ique","a"],
["ique","i"],
["ique",""],
["ique","as"],
["ique","os"],          #ttanique/ttanos
["ique","ose"],          #mtamorphique/mtamorphose
["ique","us"],              #tonique/tonus
["ique","icit"],
["ique","isme"],
    ) ;

@Regleaire = (
["ulmonaire","oumon"], 
["ulaire","le"],        # muscle/musculaire
["taire","te"],
["laire",""],           # suffixe en "aire"
["laire","e"],
["iaire","e"],
["naire",""],
["naire","in"],
["aire",""],
["aire","e"],
["aire",""],         #volontaire/volont
["aire","ation"],
    );

@Regleal = (
["oronal","ouronne"],
["estial","te"],
["arial","aire"],   	# salaire/salarial  
["dical","decine"],   	# mdical/mdecine 
["ital","et"],          # suffixe en "al"
["inal","en"],
["inal",""],   		# nominal/nom 
["ical","ex"],
["ocal","oix"],          
["oral","eur"],     #floral/fleur     
["acal","e"],          
["asal","ez"],          
["yal","i"],
["ial","e"],          # province/provincial
["cal","que"],
["al","e"],
["al",""],
["al","um"],        #rectal/rectum
["al","us"],        #viral/virus
    );

@Regleeux = (
["oureux","eur"],          
["uleux","le"],          
["ileux","oil"],    #pileux/poil
["ieux","ion"],
["ieux","e"],
["eux","e"],
["eux",""],
    ) ;

@Regleoire = (
["oire","ion"],
    ) ; 

@Regleeaigu = (
["bor", "bre"],
["ig", "ige"],
["ul", "le"],
["","us"],           #cacte/cactus
["", "e"],
["", "ure"],
    ) ;

@Reglecole = (
["cole","culture"],
    ) ;

@Regleer = (
["gulier", "gle"],
["ier","e"],            #bourse/boursier
["ier",""],
["er","e"],
    ) ; 

@Regleien = (
["idien","is"],     #rachidien/rachis
["orien","oire"],
["ien","ie"],
["ien","is"],       #pubien/pubis
["ien",""],          #Balzac/balzacien
    ) ;

@Regleain = (
["orain","oire"],
    ) ;

@Regleel = (
["iel","ie"],
["el","it"],   #eternel/eternit
["el","e"],		#culture/culturel
["el",""],
    ) ;

@Reglein = (
["arin","er"],
["alin","el"],
["guin","g"],
["lin",""],
["in","us"],               #utrin/utrus
["in",""],
    ) ;

@Regleif = (
["ursif","ours"],
["if",""],
["if","ion"],
    ) ; 


@tabSufrel = ( "ique", "aire", "al", "el", "eux", "in", "ien", "il", "estre", "ain", "if", "oire", "esque", "ois", "an", "ais", "", "cole", ) ;


@tabPrefixe = ( "ant", "anti", "archi", "auto", "arrire", "avant", "bi", "co", "contre", "de", "d", "di", "pi", "extra", "hmi", "hyper", "hypo", "il", "im", "in", "infra", "inter", "ir", "macro", "mta", "micro", "mini", "mono", "multi", "no", "non", "para", "pri", "post", "pr", "pro", "pseudo", "quasi", "re", "r", "rtro", "sans", "semi", "sous", "sub", "super",  "supra",  "sur", "tl", "trans", "tri", "ultra",) ;

#Traitement morphologique des inversions refletant "action de V"
@tabInv = ("ade", "age", "ation", "ement", "erie", "ure", "it", ) ;

%SuffInv = (
    ade =>  \@RegleIade ,
    age =>  \@RegleIage,
    ation   =>  \@RegleIation,
    ement   =>  \@RegleIement,
    ence   =>  \@RegleIence,
    erie   =>  \@RegleIerie,
    esse   =>  \@RegleIesse,
    eur   =>  \@RegleIeur,
    ise   =>  \@RegleIise,
    ion   =>  \@RegleIion,
    ure   =>  \@RegleIure,
#    t   =>  \@RegleIit,
    );

@RegleIade = (
["ade","er"],
    ) ; 

@RegleIteaigu = (
["iret","ier"],   #grossier/grossiret
["art","air"],   #clair/clart
["it","e"],      #stupide/stupidit
["it",""],
  ["t",""],  #beau/beaut
    ) ; 

@RegleIerie = (
["erie","er"],
["erie","e"],
    ) ; 

@RegleIence= (
["ence","ent"],  #transparent/transparence
    ) ; 

@RegleIesse= (
["ssesse","s"],  #bas/bassesse
["esse","e"],  #vite/vitesse
    ) ; 

@RegleIeur = (
["sseur","s"],  #gros/grosseur
["eur",""],  #lent/lenteur
    ) ; 

@RegleIise = (
["ise","e"], #bte/btise
    ) ; 

@RegleIure = (
["ure",""], #droit/droiture
["ure","er"],
    ) ; 

@RegleIion = (
["ion",""], #correct/correction
    ) ; 

@RegleIage = (
["issage","ir"],
["age","er"],
    ) ; 

@RegleIation = (
["ification",""],
["isation",""],
["ation","er"],
    ) ; 

@RegleIement = (
["chissement","c"],
["issement","ir"],
["nement","ner"],
    ) ; 



%SuffEns = (
    ade =>  \@RegleEade ,
    age =>  \@RegleEage,
    erie => \@RegleEerie,
    );

@tabEns = ("age", "ade", "erie") ;

@RegleEade = (
["ade","e"],
["ade",""],
    ) ; 

@RegleEage = (
["lage",""],
["osage","t"],
["nage",""],
["gage","gue"],   #langage/langue
["age","e"],
["age",""],
    ) ; 

@RegleEerie = (
["nnerie","n"],
["erie","e"],
["erie",""],
    ) ; 

